include(`macros.m4')
<HTML>
<HEAD>
<TITLE>"The Next Level" Desktop Configuration, Customization</TITLE>
</HEAD>
<BODY>
TOC_HEADING

<H2><A NAME="customizing">Customization</A></H2>

"The Next Level" provides some easy control for users to customize the
general look and feel of their desktop, without complicated FVWM commands.
<A HREF="#advanced">Advanced customizing</A> is another topic.<P>

There are two main files you'll need to look at to select the options
you prefer:

<H3><A NAME="fvwm2rc-defines"><TT>.fvwm2rc.defines</TT></A> file</H3>

This file specifies numerous options, all in the form of <TT>M4</TT> macros.
Be careful editing the file, since <TT>M4</TT> is very picky about quotations,
and uses matching single quotes, ` and ', unlike most other Unix programs.<P>

The <TT>.fvwm2rc.defines</TT> is thoroughly commented throughout, and you
should read those comments carefully as you edit the file.

<H3><A NAME="xdefaults-fvwm-defines"><TT>.Xdefaults-fvwm-defines</TT></A> file</H3>

Since X resources control many aspects of the visual appearance of you
desktop, you may need to edit this file to create the look and feel
you desire.  In contrast to <A
HREF="fvwm2rc-defines">.fvwm2rc.defines</A>, this file uses
<TT>cpp</TT>, the C language preprocessor, as the language for its
option definitions.  This is the default for <TT>xrdb</TT>.  Also note
that an exclamation point (<TT>!</TT>) can be used to initiate a
comment in X resource files.<P>

See <A HREF="#addapps">adding applications</A> for an example of a
fairly straightforward customization.


<HR> <H2><A NAME="advcustomizing">Advanced customization</A></H2>

"The Next Level" is a complex system.  Non-trivial customization requires
knowledge of many aspects of the Unix and X environments. You should
study all files that come with this distribution to see how they all
fit together.<P>

There are comments throughout all of the <TT>.fvwm2rc.*</TT> files which
should help explain where various things go.  As a simple overview, see
the comments at the end of <TT>.fvwm2rc.m4</TT>.  You should also study the
numerous macros as they are defined in that file.<P>

<A NAME="addapps">To add local applications</A>, you should edit
<TT>.fvwm2rc.programs</TT>.  Simple follow the patterns there.  Note
that if FVWM can't find a program you add to a menu, that menu item
won't appear.  Ensure your execution path is set properly before FVWM
is run by <TT>.xsession</TT> or <TT>.xinitrc</TT>.

<HR>
<H2><A NAME="fvwmpatch">Fvwm Patch</A></H2>

<P>
<B>What It Does</B><P>
This patch modifies the WindowList command so that if the second argument
is negative, only icons are listed in the WindowList.  Everything else
is as specified in the man page for the WindowList command.

<P>
<B>Example</B><P>

This is in <TT>.fvwm2rc.keys</TT>:

<PRE>
....
Key Return	A	CS	WindowList 1 -1
....
</PRE>

It makes gif_CS RETURN give a list of just icons, using their icon names,
of programs on any desktop.

<PRE>
....
Key Return      A       CS      WindowList 4 -2
....
</PRE>

Would make gif_CS RETURN give a list of just icons, using their window names,
of only those programs on Desktop 2 (the absolute value of -2).


<P>
<B>Applying the patch</B><P>
Do something like:

<PRE>
tar xzf /a/path/to/fvwm2-XXXX.tgz
cd fvwm2-XXXX
cd fvwm
patch < /the/patch/dir/fvwm-2.0.42-windows.c.patch
cd ..
</PRE>

Patches should succeed completely.

Then continue to build fvwm as directed.

<P>
<B>Alternatives</B><P>
If there is a way to do this without patching Fvwm, I'd love to hear
about it.  This solution may still be the best, since it's a trivial
patch, and results in an instantaneous icon menu.  The other possibilities
all involve executing a module which has much higher overhead.

<HR>
<H2><A NAME="zshpatch">Zsh Patch</A></H2>

<P>
<B>What It Does</B><P>

This patch adds a new special parameter, called POSTPROMPT, which works
like PROMPT, except it is output *after* the current command line gets
parsed and added to the history.

The patch also adds a new percent-escape sequence to those recognized by
PROMPT and POSTPROMPT, "%H", which means the last history line, after
parsing.

These two features, combined with control characters for setting the title
or icon of an xterm/nxterm, give the user the capability to put the currently
executing command in the title bar.

<P>
<B>Example</B><P>

The easiest thing to do is just use my ZShell configuration.  The pertinent
parts are:

<PRE>
setopt INTERACTIVE_COMMENTS
setopt PROMPT_SUBST
PSVAR=" "
PS1='%B%m,%l:%(6c,...%5c,%~)%#%b '
if [ "$TERM" = "xterm" -o "$TERM" = "color_xterm" ]; then
  PS1='%{]0;%m %0v%}'$PS1
  # POSTPROMPT variable requires a patch by <gjb@cs.duke.edu>
  POSTPROMPT='%{]0;%H%}'
fi
</PRE>

Where the first character inside %{ is ESC, and the last before %} is
Ctrl-G for both PS1 and POSTPROMPT.  The strings set the title bar of an
xterm.  Note that rxvt does not accept xterm control sequences.

Since "PROMPT" resets the title bar to the hostname each time a prompt
is displayed, you may want to set PSVAR="Whatever" by hand to append a note
to the title bar for when it's not executing a command.  (Or you may choose
to not set the title bar in PS1 [just remove the PS1= inside the if block,
above]).

Setting INTERACTIVE_COMMENTS allows a more descriptive name to appear in 
the title bar.  eg.

<PRE>
make all #Linux build
</PRE>

Would show the whole command line in the title bar, to make it easier to find
that window while the build is going on, while:

<PRE>
PSVAR="Linux Build"
</PRE>
Would keep a "Linux Build" message in the title bar even after the build
completed.

Setting the option PROMPT_SUBST allows you to embed variables in the PROMPTS
which get substituted at print time. eg.

<PRE>
PS1='%B%m,%l:%(6c,...%5c,%~)%#%b %{]0;${USER} on %m %0v%}'
</PRE>

Would make the title bar look something like "gjb on oz", expanding USER
to my username.


<P>
<B>Applying the patch</B><P>

<PRE>
tar xzf /a/path/to/zsh-2.5.03.tgz
cd zsh-2.5.03
cd src
patch < /the/patch/dir/zsh-2.5.03.patch
cd ..
</PRE>

Patches should succeed completely.

Then continue to build zsh as directed.

<P>
<B>Alternatives</B><P>
If there is a way to do this without this patch, I'd love to hear
about it.  Also, it'd be great if someone would contribute similar
patches for the other shells.  ZSh is great, but not everyone uses
it.

ADDRESS_FOOTER

</BODY>
</HTML>
